function [ data, g, data0 ] = ...
  constantLinearImplicit(linear_constant, input_bounds, dynamics_term)
% constantLinearImplicit: constant linear reach set (regular 2D HJ PDE)
%
%   [ data, g, data0 ] = ...
%     constantLinearImplicit(linear_constant, input_bounds, dynamics_term)
%  
% This is a standard reachability calculation for the constant linear
% example, which is a variation on the double integrator, with additional
% terms in the integrating variable's dynamics: (1) an constant linear term
% and (2) an input.  Because of these additional terms, the terminology
% "position" and "velocity" for the two variables don't make quite as much
% sense anymore.
%
%     \dot x_1 = a * x_1 + x_2 + v  (the horizontal direction; 'position')
%     \dot x_2 = u                  (the vertical direction; 'velocity')
%
%       with a given by linear_constant 
%       and bounds on the inputs u_* given by input_bounds
%
% with both constant k defined by input parameter linear_constant and
% inputs bounded in magnitude by input parameter input_bounds.
%
% The default settings (1 & [ 0.5, 0.25 ]) are chosen to correspond to
% an example which is supported by HJ PDE theory, but has unstable forward
% linear dynamics.  An example which is not supported by HJ PDE theory but
% is forward stable is (-1 & [ 0.0, 0.1 ]).  In both cases the reachable
% sets are noticably different than the regular double integrator and each
% other, but they are non-trivial (eg: not empty or the same as the target
% set).
%
% There are many options in the function which can only be modifed by
% changing the file.
%
% Input Parameters:
%
%   linear_constant: Double.  Negative values will correspond to systems
%   that are forward stable.  Non-negative values are required for the
%   viscosity solution of the HJ PDE to be well defined in backward
%   reachability.  Small values make the system more like a traditional
%   double integrator.  Optional.  Default is 1.0.
%
%   input_bounds: Two element non-negative double array.  Bounds on the
%   input signals in the (non-intuitive) order [ v, u ]. Optional.  Default
%   is [ 0.5, 0.25 ].
%
%   dynamics_term: String.  Specifies how the reach set should be
%   calculated.  Optional.  Default = 'lxf'.  Two options are available:
%
%     'convection': Use feedback inputs v = -sign(x_1) and u = -sign(x_2)
%     to precompute the vector field, and then determine the reachable set
%     using an upwinded advection scheme.  Note that this choice of
%     feedback input is optimal for u but suboptimal for v.
%
%     'lxf': Use optimal feedback inputs as computed by general HJ PDE, and
%     then determine the reachable set using a Lax-Friedrichs scheme.  Note
%     that this approach will introduce more numerical dissipation.
%
% Output Parameters:
%
%   data: Implicit surface function at t_max.
%
%   g: Grid structure on which data was computed.
%
%   data0: Implicit surface function at t_0.

% Copyright 2009 Ian M. Mitchell (mitchell@cs.ubc.ca).
% This software is used, copied and distributed under the licensing 
%   agreement contained in the file LICENSE in the top directory of 
%   the distribution.
%
% Created by Ian Mitchell, 9/21/09
% Subversion tags for version control purposes.
% $Date: 2011-04-11 10:47:05 -0700 (Mon, 11 Apr 2011) $
% $Id: constantLinearImplicit.m 581 2011-04-11 17:47:05Z mitchell $

  %---------------------------------------------------------------------------
  % You will see many executable lines that are commented out.
  %   These are included to show some of the options available; modify
  %   the commenting to modify the behavior.

  %---------------------------------------------------------------------------
  %% Make sure we can see the kernel m-files.
  run('addPathToKernel');

  %---------------------------------------------------------------------------
  %% Optional input parameters.

  if(nargin < 1)
    linear_constant = +1;
  end

  if(nargin < 2)
    input_bounds = [ +0.5, +0.25 ];
  end

  if(nargin < 3)
    dynamics_term = 'lxf';
  end

  %---------------------------------------------------------------------------
  %% PDE solver parameters.

  % Integration parameters.
  tMax = 4.0;                  % End time.
  plotSteps = 9;               % How many intermediate plots to produce?
  t0 = 0;                      % Start time.
  singleStep = 0;              % Plot at each timestep (overrides tPlot).

  % Period at which intermediate plots should be produced.
  tPlot = (tMax - t0) / (plotSteps - 1);

  % How close (relative) do we need to get to tMax to be considered finished?
  small = 100 * eps;

  % Pause after each plot?
  pauseAfterPlot = 0;

  % Delete previous plot before showing next?
  deleteLastPlot = 0;

  % Plot in separate subplots (set deleteLastPlot = 0 in this case)?
  useSubplots = 0;

  % Parameters for visualizeLevelSets.
  level = 0;
  displayType = 'contour';

  %---------------------------------------------------------------------------
  %% Optional and problem specific parameters.

  % What kind of target set?
  target_set_type = 'rectangle'; %#ok<NASGU>
  target_set_type = 'hexagon';

  % Compute with both positive and negative velocities, or only positive?
  negative_velocities = 1;

  % Should we only allow the reach set to grow?
  restrict_update = 1;

  % Spatial approximation scheme.
  derivFunc = @upwindFirstENO2;

  % Temporal approximation scheme.
  integratorFunc = @odeCFL2;

  % Some integration parameters apply to any integrator.
  integratorOptions = odeCFLset('factorCFL', 0.9, 'stats', 'on');
  if(singleStep)
    integratorOptions = odeCFLset(integratorOptions, 'singleStep', 'on');
  end

  %---------------------------------------------------------------------------
  %% Create the grid.  
  g.dim = 2;
  if negative_velocities
    g.min = 1.5 * [ -1; -1 ];
  else
    g.min = 1.5 * [ -1;  0 ]; %#ok<UNRCH>
  end
  g.max = 1.5 * [ +1; +1 ];
  g.N = 151;
  g.bdry = @addGhostExtrapolate;
  g = processGrid(g);

  %---------------------------------------------------------------------------
  %% Create initial conditions.
  switch(target_set_type)

    case 'hexagon'
      % For this test, the target set is anything OUTSIDE of a hexagonal shape.
      % We use a hexagon instead of a square so that the maximum (and minimum)
      % position (y variable) for the target set is a continuous function of the 
      % velocity (x variable).  This continuity is not required for this Implicit
      % formulation, but will be important for the MIE formulation.

      % First, construct the hexagon.  Start with a rectangle to create the two
      % vertical sides.  If you just want a rectangular target set, you need not
      % include the extra intersections.
      data = shapeRectangleByCenter(g, 0, [ 2; 3 ]);
      data = shapeIntersection(data, shapeHyperplane(g, [ +1; +1 ], [ 0; +1.5 ]));
      data = shapeIntersection(data, shapeHyperplane(g, [ -1; +1 ], [ 0; +1.5 ]));
      data = shapeIntersection(data, shapeHyperplane(g, [ +1; -1 ], [ 0; -1.5 ]));
      data = shapeIntersection(data, shapeHyperplane(g, [ -1; -1 ], [ 0; -1.5 ]));

    case 'rectangle'
      % The rectangular target set cannot be done using the MIE formulation,
      % but it works fine for this Implicit formulation.
      data = shapeRectangleByCorners(g, [ -1.0; -1.0 ], [ +1.0, +1.0 ]);

    otherwise
      error('Unknown target_set_type: %s', target_set_type);
  end

  % Then take the complement, since the outside of the set is the actual
  % target for the backwards reachability.
  data = -data;
  % If necessary, provide the IC as an output.
  if(nargout > 2)
    data0 = data;
  end

  %---------------------------------------------------------------------------
  %% Set up PDE term that implements the dynamics.

  switch(dynamics_term)
    case 'convection'
      % I'm not sure if I am really guessing the feedback control for u1
      % totally wrong, or if I am implementing it wrong.  Either way, the
      % reachable set computed with this input is way different than the
      % set computed with LaxFriedrichs.
      warning('This option is buggy'); %#ok<WNTAG>
      % Given feedback inputs, a corresponding vector field can be
      % determined and implemented as a purely convectional flow field in
      % both dimensions.  The "constant linear" term is a purely convective
      % term in this Implicit formulation.  Remember to reverse the
      % dynamics (multiply by -1) since we solve the PDE backward in time.
      u1_input = input_bounds(1) * sign(g.xs{1}); %#ok<NASGU>
      u1_input = input_bounds(1) * sign(g.xs{1} - g.xs{2});
      schemeFunc = @termConvection;
      schemeData.grid = g;
      schemeData.derivFunc = derivFunc;
      schemeData.velocity = { -(linear_constant * g.xs{1} + g.xs{2} - u1_input); ...
                              -(-input_bounds(2) * sign(g.xs{2})) };

    case 'lxf'
      % A Lax-Friedrichs implementation can compute the optimal feedback
      % inputs although it is likely to be more dissipative numerically and
      % hence less accurate.
      schemeFunc = @termLaxFriedrichs;
      schemeData.derivFunc = derivFunc;
      schemeData.hamFunc = @hamFunc;
      schemeData.partialFunc = @partialFunc;
      schemeData.dissFunc = @artificialDissipationGLF;
      schemeData.grid = g;
      schemeData.linear_constant = linear_constant;
      schemeData.input_bounds = input_bounds;

    otherwise
      error('Unknown dynamics_term: %s', dynamics_term);
  end

  %---------------------------------------------------------------------------
  %% Restrict the Hamiltonian so that reachable set only grows.
  % Whatever scheme handles the dynamics must already be completely set up.
  if(restrict_update)
    innerFunc = schemeFunc;
    innerData = schemeData;
    clear schemeFunc schemeData;

    % Wrap the true Hamiltonian inside the term approximation restriction routine.
    schemeFunc = @termRestrictUpdate;
    schemeData.innerFunc = innerFunc;
    schemeData.innerData = innerData;
    schemeData.positive = 0;
  end

  %---------------------------------------------------------------------------
  %% Initialize Display
  f = figure; %#ok<NASGU>

  % Set up subplot parameters if necessary.
  if(useSubplots)
    rows = ceil(sqrt(plotSteps));
    cols = ceil(plotSteps / rows);
    plotNum = 1;
    subplot(rows, cols, plotNum);
  end

  h = visualizeLevelSet(g, data, displayType, level, [ 't = ' num2str(t0) ]);
  set(h, 'LineStyle', '--', 'LineWidth', 1, 'LineColor', 'blue');
  xlabel('position');  ylabel('velocity');

  hold on;
  if(g.dim > 1)
    axis(g.axis);
    daspect([ 1 1 1 ]);
  end

  %---------------------------------------------------------------------------
  %% Loop until tMax (subject to a little roundoff).
  tNow = t0;
  startTime = cputime;
  while(tMax - tNow > small * tMax)

    % Reshape data array into column vector for ode solver call.
    y0 = data(:);

    % How far to step?
    tSpan = [ tNow, min(tMax, tNow + tPlot) ];

    % Take a timestep.
    [ t y ] = feval(integratorFunc, schemeFunc, tSpan, y0,...
                    integratorOptions, schemeData);
    tNow = t(end);

    % Get back the correctly shaped data array
    data = reshape(y, g.shape);

    if(pauseAfterPlot)
      % Wait for last plot to be digested.
      pause;
    end

    % Delete last visualization if necessary.
    if(deleteLastPlot)
      delete(h);
    end

    % Move to next subplot if necessary.
    if(useSubplots)
      plotNum = plotNum + 1;
      subplot(rows, cols, plotNum);
    end

    % Create new visualization.
    h = visualizeLevelSet(g, data, displayType, level, [ 't = ' num2str(t) ]);
    set(h, 'LineStyle', ':', 'LineWidth', 1, 'LineColor', 'black');
    daspect([ 1 1 1 ]);

  end

  %---------------------------------------------------------------------------
  %% Termination code.  
  % Let the user know how long it all took.
  endTime = cputime;
  fprintf('Total execution time %g seconds\n', endTime - startTime);

  % Make the final contour a little more visible.
  set(h, 'LineStyle', '-', 'LineWidth', 2, 'LineColor', 'red');

  title(sprintf('Implicit: t = %5.2f, a = %4.2f, v = %4.2f, u = %4.2f', ...
                tNow, linear_constant, input_bounds(1), input_bounds(2)));
  
end % main function.


%---------------------------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%---------------------------------------------------------------------------
function hamValue = hamFunc(~, ~, deriv, schemeData)
% hamFunc: analytic Hamiltonian.
%
% hamValue = hamFunc(t, data, deriv, schemeData)
%
% This function implements the hamFunc prototype for the Implicit
% reachability version of the constant linear with input variation on the
% double integrator.
%
% Input parameters:
%
%   t: Time at beginning of timestep.
%
%   data: Data array.
%
%   deriv: Cell vector of the costate (\grad \phi).
%
%   schemeData: A structure (see below).
%
% Output parameters:
%
%   hamValue: The analytic hamiltonian.
%
% schemeData is a structure containing data specific to this Hamiltonian
% For this function it contains the field(s):
%
%   .grid: Grid structure.
%
%   .linear_constant: Double.  Constant multiplier for the linear term of
%   the integrator.
%
%   .input_bounds: Two element non-negative double array.  Bounds on the
%   input signals.

% Ian Mitchell 2010/09/07

  checkStructureFields(schemeData, 'grid', 'linear_constant', 'input_bounds');

  grid = schemeData.grid;

  % Remember to reverse the dynamics (multiply by -1) since we solve the
  % PDE backward in time.
  hamValue = -(deriv{1} .* (schemeData.linear_constant * grid.xs{1} + grid.xs{2}) ...
               + abs(deriv{1}) * schemeData.input_bounds(1) ...
               + abs(deriv{2}) * schemeData.input_bounds(2));

end % function hamFunc().


%---------------------------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%---------------------------------------------------------------------------
function alpha = partialFunc(~, ~, ~, ~, schemeData, dim)
% partialFunc: Hamiltonian partial function.
%
% alpha = partialFunc(t, data, derivMin, derivMax, schemeData, dim)
%
% This function implements the partialFunc prototype for the Implicit
% reachability version of the constant linear with input variation on the
% double integrator.
%
% It calculates the extrema of the absolute value of the partials of the 
% analytic Hamiltonian with respect to the costate (gradient).
%
% Parameters:
%
%   t: Time at beginning of timestep.
%
%   data: Data array.
%
%   derivMin: Cell vector of minimum values of the costate (\grad \phi).
%
%   derivMax: Cell vector of maximum values of the costate (\grad \phi).
%
%   schemeData: A structure (see below).
%
%   dim: Dimension in which the partial derivatives is taken.
%
% Output Parameters:
%
%   alpha: Maximum absolute value of the partial of the Hamiltonian with
%   respect to the costate in dimension dim for the specified range of
%   costate values (O&F equation 5.12). Note that alpha can (and should) be
%   evaluated separately at each node of the grid.
%
% schemeData is a structure containing data specific to this Hamiltonian
% For this function it contains the field(s):
%
%   .grid: Grid structure.
%
%   .linear_constant: Double.  Constant multiplier for the linear term of
%   the integrator.
%
%   .input_bounds: Two element non-negative double array.  Bounds on the
%   input signals.

% Ian Mitchell 2010/09/07

  checkStructureFields(schemeData, 'grid', 'linear_constant', 'input_bounds');

  grid = schemeData.grid;

  switch dim
    case 1
      alpha = (abs(schemeData.linear_constant * grid.xs{1} + grid.xs{2}) ...
               + schemeData.input_bounds(1));

    case 2
      alpha = schemeData.input_bounds(2);

    otherwise
      error('Partials do not exist in dimension %d', dim);
  end
  
end % function partialFunc().
