function obliviousHSCC(which_fig)
% obliviousHSCC: Create figures for HSCC submission.
%
% Up to two sets of figures are generated depending on the input parameter.
%
%   1) Projections of the reach tube onto (x, vp, psi) and (y, vp, psi) for
%   each of full dimensional implicit, decoupled implicit and MIE
%   formulations, for a total of six figures.
%
%   2) Slices of the reach tube in (x, y, psi) space for a fixed value of
%   vp for the full dimensional implicit and decoupled implicit data.
%
% This code was copied from obliviousImplicitVisualize and
% oblivousCompareDecoupled and then modified to produce figures appropriate
% to the HSCC submission.
%
% Input Parameters:
%
%   which_fig: String.  Optional.  Default = 'all'.  Choices are
%
%     'projections': The projections of the reach tube, item (1) above.
%
%     'slices': The slices of the reach tube, item (2) above.
%
%     'all': All sets of figures.

% Copyright 2011 Ian M. Mitchell (mitchell@cs.ubc.ca).
% This software is used, copied and distributed under the licensing 
%   agreement contained in the file LICENSE in the top directory of 
%   the distribution.
%
% Created by Ian Mitchell, 2011/01/30.
% Subversion tags for version control purposes.
% $Date: 2011-04-12 20:10:31 -0500 (Tue, 12 Apr 2011) $
% $Id: obliviousHSCC.m 582 2011-04-13 01:10:31Z mitchell $

  %------------------------------------------------------------------------
  %% Parameters
  
  if(nargin < 1)
    which_fig = 'all';
  end

  % What is the beginning of the error message identifier if the data files
  % are not found?
  file_not_found_error_id = 'MATLAB:load';
  
  % Where are the dimensions in the 3D grids?
  s2i_3D = struct('xy', 1, 'vp', 2, 'psi', 3);
  
  % Where are the dimensions in the 4D grid?
  s2i_4D = struct('x', 1, 'y', 2, 'vp', 3, 'psi', 4);
  
  % What level set(s)?
  level = 0;

  % Viewing angles
  x_view = { 37.5, 15 };
  y_view = { 37.5, 15 };
  slice_view = { 60, 15 };

  % What slice to view?
  slice_vp = 2.0;
  
  % What filenames to use?
  projection_base_filename = 'oblivious_projections';
  slice_base_filename = 'oblivious_slices';
  
  %------------------------------------------------------------------------
  %% Get the data.
  try
    % It is assumed that the grids used for all computations are compatible
    % (they have exactly the same bounds and dx). Consequently, this code
    % only needs to work with the 3D grid.
    all_implicit = load('obliviousImplicit_all.mat');
    g_all = processGrid(all_implicit.g);
    x_implicit = load('obliviousImplicit_x.mat');
    g_x = processGrid(x_implicit.g);
    y_implicit = load('obliviousImplicit_y.mat');
    g_y = processGrid(y_implicit.g);
    mie = load('obliviousMIE.mat');
  catch ME
    if strncmpi(ME.identifier, file_not_found_error_id, length(file_not_found_error_id))
      % Assume that one or more of the input files were missing.  Out of
      % coding laziness, we'll rerun them all.
      fprintf('\nRegenerating missing data files (this may take a while).\n\n');
      [ all_implicit.data, g_all ] = obliviousImplicit('all');
      % Remove the enormous and easy to recreate xs field from the grid
      % structure before saving it.  We need to use the variable names 'g'
      % and 'data' so that the files' contents will be reloaded into the
      % correct variables the next time we recreate the figures.
      g = rmfield(g_all, 'xs'); %#ok<NASGU>
      data = all_implicit.data; %#ok<NASGU>
      save('obliviousImplicit_all.mat', 'data', 'g');

      % Repeat this process for the other three files.
      [ x_implicit.data, g_x ] = obliviousImplicit('x_only');
      g = rmfield(g_x, 'xs'); %#ok<NASGU>
      data = x_implicit.data; %#ok<NASGU>
      save('obliviousImplicit_x.mat', 'data', 'g');
      [ y_implicit.data, g_y ] = obliviousImplicit('y_only');
      g = rmfield(g_y, 'xs'); %#ok<NASGU>
      data = y_implicit.data; %#ok<NASGU>
      save('obliviousImplicit_y.mat', 'data', 'g');
      [ mie.soln, g ] = obliviousMIE;
      g = rmfield(g, 'xs'); %#ok<NASGU>
      soln = mie.soln; %#ok<NASGU>
      save('obliviousMIE.mat', 'soln', 'g');
      
      % Clear the dummy data, soln and g variables to make sure that they
      % are not used inadvertently.
      clear data g soln
    else
      % Some other error occurred, so just pass it along.
      rethrow(ME);
    end
  end

  %------------------------------------------------------------------------
  %------------------------------------------------------------------------
  %% Create the projection figures.
  if(strcmpi(which_fig, 'projections') || strcmpi(which_fig,'all'))

    %% Project the full dimensional data into appropriate subspaces.

    % Apparently the min operation does not reduce the dimension of an
    % array, it merely reduces the corresponding dimension to a single
    % element.
    x_full.data = squeeze(min(all_implicit.data, [], s2i_4D.y));
    y_full.data = squeeze(min(all_implicit.data, [], s2i_4D.x));
  
    %------------------------------------------------------------------------
    %% Create implicit surface versions of MIE data on 3D grid.
  
    % First, we need to extend the MIE data onto the 3D grid.  The added
    % dimension will actually be the first dimension, since the MIE grid is
    % in dimensional order [ vp, psi ] and the implicit grid is in
    % dimensional order [ (x or y), vp, psi ].

    % The permutation that goes from the implicit grid's dimensional ordering
    % to an MIE grid (replicated in the position dimension).  We will use the
    % inverse of this permutation on the MIE data.
    perm = [ s2i_3D.vp, s2i_3D.psi, s2i_3D.xy ];
  
    % Replicate the MIE data in the (added) position dimension, and then
    % permute to get it into the dimensional order for the implicit grid.
    mie.soln.x_max_3D = ipermute(repmat(mie.soln.x_max, [ 1 1 g_x.N(s2i_3D.xy) ]), perm);
    mie.soln.x_min_3D = ipermute(repmat(mie.soln.x_min, [ 1 1 g_x.N(s2i_3D.xy) ]), perm);
    mie.soln.y_max_3D = ipermute(repmat(mie.soln.y_max, [ 1 1 g_y.N(s2i_3D.xy) ]), perm);
    mie.soln.y_min_3D = ipermute(repmat(mie.soln.y_min, [ 1 1 g_y.N(s2i_3D.xy) ]), perm);
  
  
    % These are NOT signed distance functions, particularly near the edges of
    % the grid, but hopefully they are nice enough for visualization
    % purposes.
    x_mie.data = max(g_x.xs{s2i_3D.xy} - mie.soln.x_max_3D, mie.soln.x_min_3D - g_x.xs{s2i_3D.xy});
    y_mie.data = max(g_y.xs{s2i_3D.xy} - mie.soln.y_max_3D, mie.soln.y_min_3D - g_y.xs{s2i_3D.xy});

    %------------------------------------------------------------------------
    %% Visualize.
    
    figure;
    x_full.h = visualizeLevelSet(g_x, x_full.data, 'surface', level);
    view(x_view{:});
    prettyProjectionFigure('Full Dimensional Implicit Formulation', 'x_1');
    png_filename = sprintf('%s_%s_%s.png', projection_base_filename, 'full', 'x');
    print('-dpng', png_filename);
    print('-deps', sprintf('%s_%s_%s.eps', projection_base_filename, 'full', 'x'));

    figure;
    x_implicit.h = visualizeLevelSet(g_x, x_implicit.data, 'surface', level);
    view(x_view{:});
    prettyProjectionFigure('Decoupled Implicit Formulation', 'x_1');
    png_filename = sprintf('%s_%s_%s.png', projection_base_filename, 'decoupled', 'x');
    print('-dpng', png_filename);
    print('-deps', sprintf('%s_%s_%s.eps', projection_base_filename, 'decoupled', 'x'));

    figure;
    x_mie.h = visualizeLevelSet(g_x, x_mie.data, 'surface', level);
    view(x_view{:});
    prettyProjectionFigure('MIE Formulation', 'x_1');
    png_filename = sprintf('%s_%s_%s.png', projection_base_filename, 'mie', 'x');
    print('-dpng', png_filename);
    print('-deps', sprintf('%s_%s_%s.eps', projection_base_filename, 'mie', 'x'));

    figure;
    y_full.h = visualizeLevelSet(g_y, y_full.data, 'surface', level);
    view(y_view{:});
    prettyProjectionFigure('Full Dimensional Implicit Formulation', 'x_2');
    png_filename = sprintf('%s_%s_%s.png', projection_base_filename, 'full', 'y');
    print('-dpng', png_filename);
    print('-deps', sprintf('%s_%s_%s.eps', projection_base_filename, 'full', 'y'));

    figure;
    y_implicit.h = visualizeLevelSet(g_y, y_implicit.data, 'surface', level);
    view(y_view{:});
    prettyProjectionFigure('Decoupled Implicit Formulation', 'x_2');
    png_filename = sprintf('%s_%s_%s.png', projection_base_filename, 'decoupled', 'y');
    print('-dpng', png_filename);
    print('-deps', sprintf('%s_%s_%s.eps', projection_base_filename, 'decoupled', 'y'));

    figure;
    y_mie.h = visualizeLevelSet(g_y, y_mie.data, 'surface', level);
    view(y_view{:});
    prettyProjectionFigure('MIE Formulation', 'x_2');
    png_filename = sprintf('%s_%s_%s.png', projection_base_filename, 'mie', 'y');
    print('-dpng', png_filename);
    print('-deps', sprintf('%s_%s_%s.eps', projection_base_filename, 'mie', 'y'));

  end % Projections figures.
  
  %------------------------------------------------------------------------
  %------------------------------------------------------------------------
  %% Create the slice figures.
  if(strcmpi(which_fig, 'slices') || strcmpi(which_fig,'all'))

    %------------------------------------------------------------------------
    %% Generate the full dimensional slice.
    
    % Identify slice index in full dimensional grid.
    [ ~, all_implicit.slice_index ] = min(abs(g_all.vs{s2i_4D.vp} - slice_vp));

    % Create a cell vector of indexes to pick out that slice.
    all_implicit.slice_indexes = cell(g_all.dim, 1);
    for d = 1 : g_all.dim
      all_implicit.slice_indexes{d} = 1 : g_all.N(d);
    end
    all_implicit.slice_indexes{s2i_4D.vp} = all_implicit.slice_index;
    
    % Generate a figure of that slice.
    figure;
    all_implicit.h = visualizeLevelSet(deleteDimensionGrid(g_all, s2i_4D.vp), ...
                                       squeeze(all_implicit.data(all_implicit.slice_indexes{:})), ...
                                       'surface', level);
    view(slice_view{:});
    prettySliceFigure('Full Dimensional Implicit Formulation', ...
                      g_all.vs{s2i_4D.vp}(all_implicit.slice_index));
    png_filename = sprintf('%s_%s.png', slice_base_filename, 'full');
    print('-dpng', png_filename);
    print('-deps', sprintf('%s_%s.eps', slice_base_filename, 'full'));
    
    %------------------------------------------------------------------------
    %% Generate the decoupled slice.
    
    % Extract the slice of the x and y data of interest.
    [ ~, x_implicit.slice_index ] = min(abs(g_x.vs{s2i_3D.vp} - slice_vp));
    [ ~, y_implicit.slice_index ] = min(abs(g_y.vs{s2i_3D.vp} - slice_vp));
    x_implicit.slice_value = g_x.vs{s2i_3D.vp}(x_implicit.slice_index);
    y_implicit.slice_value = g_y.vs{s2i_3D.vp}(y_implicit.slice_index);
    if(x_implicit.slice_value ~= y_implicit.slice_value)
      error('Cannot backproject for slices that do not match');
    end
    
    % Create cell vectors of indexes to pick out that slice.
    x_implicit.slice_indexes = cell(g_x.dim, 1);
    for d = 1 : g_x.dim
      x_implicit.slice_indexes{d} = 1 : g_x.N(d);
    end
    x_implicit.slice_indexes{s2i_3D.vp} = x_implicit.slice_index;
    
    y_implicit.slice_indexes = cell(g_y.dim, 1);
    for d = 1 : g_y.dim
      y_implicit.slice_indexes{d} = 1 : g_y.N(d);
    end
    y_implicit.slice_indexes{s2i_3D.vp} = y_implicit.slice_index;
    
    % Extract the slices
    x_slice.data_2D = squeeze(x_implicit.data(x_implicit.slice_indexes{:}));
    y_slice.data_2D = squeeze(y_implicit.data(y_implicit.slice_indexes{:}));

    % Backproject into 3D by replicating in appropriate dimension.  At this
    % point I am hardcoding the dimension ordering.
    x_slice.data_3D = repmat(reshape(x_slice.data_2D, [ g_x.N(s2i_3D.xy), 1, g_x.N(s2i_3D.psi) ]), [ 1, g_y.N(s2i_3D.xy), 1 ]);
    y_slice.data_3D = repmat(reshape(y_slice.data_2D, [ 1, g_y.N(s2i_3D.xy), g_y.N(s2i_3D.psi) ]), [ g_x.N(s2i_3D.xy), 1, 1 ]);
    backproject.data = shapeIntersection(x_slice.data_3D, y_slice.data_3D);

    % It is convenient to have a grid structure for this backprojection so
    % that we can use visualizeLevelSet().  Unfortunately, simply deleting
    % a dimension from the full dimensional calculation won't work because
    % it was done at a lower resolution.
    backproject.g.dim = 3;
    backproject.g.N = [ g_x.N(s2i_3D.xy); g_y.N(s2i_3D.xy); g_x.N(s2i_3D.psi) ];
    backproject.g.min = [ g_x.min(s2i_3D.xy); g_y.min(s2i_3D.xy); g_x.min(s2i_3D.psi) ];
    backproject.g.max = [ g_x.max(s2i_3D.xy); g_y.max(s2i_3D.xy); g_x.max(s2i_3D.psi) ];
    % The boundary condition fields are irrelevant, since we will not
    % compute with this grid, and the other fields will be filled in by
    % processGrid.
    backproject.g = processGrid(backproject.g);
    
    % Generate a figure of that slice.
    figure;
    backproject.h = visualizeLevelSet(backproject.g, backproject.data, 'surface', level);
    view(slice_view{:});
    prettySliceFigure('Decoupled Implicit Formulation', x_implicit.slice_value);
    png_filename = sprintf('%s_%s.png', slice_base_filename, 'decoupled');
    print('-dpng', png_filename);
    print('-deps', sprintf('%s_%s.eps', slice_base_filename, 'decoupled'));
    
  end % Slices figures.
  
end % obliviousHSCC().

    
%---------------------------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%---------------------------------------------------------------------------
function prettyProjectionFigure(title_string, position_variable)
% prettyProjectionFigure: Make a projection figure pretty.

  xlabel(position_variable);  ylabel('v_p');  zlabel('\theta');
  title(title_string);
  camlight headlight;
  axis tight;
  daspect([ 5 1 2 ]);

end % prettyProjectionFigure().

%---------------------------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%---------------------------------------------------------------------------
function prettySliceFigure(title_string, slice_location)
% prettySliceFigure: Make a slice figure pretty.

  xlabel('x_1');  ylabel('x_2');  zlabel('\theta');
  title(sprintf('%s at v_p = %4.2f', title_string, slice_location));
  camlight headlight;
  axis tight;
  daspect([ 5 5 2 ]);

end % prettySliceFigure().

