function [ ydot, stepBound, schemeData ] = termVectorLaxFriedrichs(t, y, schemeData)
% termVectorLaxFriedrichs: approximate H(x,p) term in a vector HJ PDE with Lax-Friedrichs.
%
%   [ ydot, stepBound, schemeData ] = termVectorLaxFriedrichs(t, y, schemeData)
%
% Computes a Lax-Friedrichs (LF) approximation of a general vector
% Hamilton-Jacobi equation.  Global LF, Local LF, Local Local LF and
% Stencil LF are implemented by choosing different dissipation functions.
% The PDE is:
%
%            D_t \phi = -H(x, t, \phi, D_x \phi).
%
% Based on methods outlined in O&F, chapter 5.3 and 5.3.1.
%
% The purpose of this routine is for vector HJ PDEs whose vector
% components' equations are so closely coupled that the computation of one
% requires gradient information about the others.  The vector level set
% scheme from ToolboxLS does not easily permit such close coupling, since
% it was designed to decouple the elements of the vector computation as
% much as possible (to encourage reuse of the standard, scalar PDE term
% approximation routines).  In this routine, all components of the vector
% PDE are computed at once by a single call to the Hamiltonian, which has
% access to all the gradients.
%
% In order to implement this coordinated computation, the elements of the
% vector solution cannot be stored as a cell vector as ToolboxLS normally
% does for vector level sets.  Instead, a single data array with one
% dimension more than the grid is created.  The final dimension of this
% data array has one slice for each element of the vector solution.
% Likewise, when the gradient approximations are computed, their data
% arrays also contain one extra dimension.
%
% This unusual storage structure for the data and gradients must be taken
% into account in the main routine, the Hamiltonian, and possibly the
% partial derivative bounding function.
%
% Input Parameters:
%
%   t: Time at beginning of timestep.
%
%   y: Data array in vector form.
%
%   schemeData: A structure (see below).
%
% Output Parameters:
%
%   ydot: Change in the data array, in vector form.
%
%   stepBound: CFL bound on timestep for stability.
%
%   schemeData: The input structure, possibly modified.
%
% schemeData is a structure containing data specific to this type of 
% term approximation.  For this function it contains the field(s)
%
%   .grid: Grid structure (see processGrid.m for details).  Note that the
%   data array will be multiple copies of this grid, concatenated along a
%   new additional dimension.
%
%   .derivFunc: Function handle to upwinded finite difference derivative
%   approximation.
%
%   .dissFunc: Function handle to LF dissipation calculator.
%
%   .hamFunc: Function handle to analytic hamiltonian H(x,p).
%
%   .partialFunc: Function handle to extrema of \partial H(x,p) / \partial p.
%
% Note that options for derivFunc and dissFunc are provided as part of the
% level set toolbox, while hamFunc and partialFunc depend on the exact term
% H(x,p) and are user supplied.  Note also that schemeData may contain
% addition fields at the user's discretion; for example, fields containing
% parameters useful to hamFunc or partialFunc.
%
% schemeData.hamFunc should have one of the prototypes
%
%         [ hamValue ]             = hamFunc(t, data, deriv, schemeData)
%         [ hamValue, schemeData ] = hamFunc(t, data, deriv, schemeData)
%
% where t and schemeData are passed directly from this function, data = y
% has been reshaped into its original size, and deriv is a cell vector (of
% length grid.dim) containing the elements of the costate p = \grad \phi.
% The return value should be an array (the size of data) containing H(x,p).
% Optionally, a modified schemeData structure may also be returned. Any
% modifications to the schemeData structure will be visible immediately to
% schemeData.partialFunc on this timestep.
%
% For details on schemeData.partialFunc, see the dissipation functions.


% Copyright 2010 Ian M. Mitchell (mitchell@cs.ubc.ca).
% This software is used, copied and distributed under the licensing 
%   agreement contained in the file LICENSE in the top directory of 
%   the distribution.
%
% Created by Ian Mitchell, 2010/10/28.
% Subversion tags for version control purposes.
% $Date: 2011-04-12 20:10:31 -0500 (Tue, 12 Apr 2011) $
% $Id: termVectorLaxFriedrichs.m 582 2011-04-13 01:10:31Z mitchell $

  %------------------------------------------------------------------------
  %% Some checks on the input

  % We treat vector level sets differently than the rest of ToolboxLS.
  if(iscell(schemeData))
    error('Traditional vector level sets are not supported by this routine.');
  end
  
  checkStructureFields(schemeData, 'grid', 'derivFunc', 'dissFunc', ...
                                       'hamFunc', 'partialFunc');
  grid = schemeData.grid;

  %------------------------------------------------------------------------
  %% Reshape data array.
  % Note that the data array is actually a concatenation of multiple data
  % arrays, one for each component of the vector equation.  Each component
  % lives on the grid.  Consequently, the data we received is actually from
  % an array of one higher dimension than the grid, and we need to figure
  % out the size of that dimension.
  vector_length = numel(y) / prod(grid.N);
  if(vector_length ~= round(vector_length))
    % The size of the data that we received is not a multiple of the size
    % of the grid, so it makes no sense as a concatenation of grid data
    % arrays.
    error('Data array is not of the correct size.');
  end

  % We need to use grid.N here rather than grid.shape, because for 1D grids
  % we store the components in the second dimension.
  vector_shape = [ grid.N', vector_length ];
  data = reshape(y, vector_shape);

  %------------------------------------------------------------------------
  %% Get upwinded and centered derivative approximations
  
  % We only compute derivatives across the grid's dimensions, not the added
  % dimension that handles the vector of solutions.  However, since the
  % derivative approximation routine requires that the data and the grid
  % correspond, we need to do this computation separately for each element
  % of the vector solution.
  derivL = cell(grid.dim, 1);
  derivR = cell(grid.dim, 1);
  derivC = cell(grid.dim, 1);

  % Yes, this index cell vector does include every single element in a
  % regular grid array.  However, we use it to ensure that the indexing for
  % the component of the vector solution (loop variable j below) appears in
  % the right spot.
  index = cell(grid.dim, 1);
  for i = 1 : grid.dim
    index{i} = 1 : grid.N(i);
  end

  % Compute left, right and center derivatives in every grid dimension for
  % every component of the vector solution.
  for i = 1 : grid.dim
    for j = 1 : vector_length
      [ derivL{i}(index{:}, j), derivR{i}(index{:}, j) ] ...
        = feval(schemeData.derivFunc, grid, squeeze(data(index{:}, j)), i);
    end
    % We can do all of the components of the centered approximation
    % together, since it does not involve accessing the grid.
    derivC{i} = 0.5 * (derivL{i} + derivR{i});
  end

  %------------------------------------------------------------------------
  %% Lax-Friedrichs scheme.
  
  % Analytic Hamiltonian with centered difference derivatives.
  switch(nargout(schemeData.hamFunc))
    case 1
      % No change to schemeData.
      ham = feval(schemeData.hamFunc, t, data, derivC, schemeData);
    case 2
      [ ham, schemeData ] = ...
        feval(schemeData.hamFunc, t, data, derivC, schemeData);
    otherwise
      error('Cannot handle hamFunc returning %d arguments', nargout(schemeData.hamFunc));
  end

  % Lax-Friedrichs dissipative stabilization.
  [ diss, stepBound ] = feval(schemeData.dissFunc, t, data, derivL, derivR, schemeData);
  
  % Calculate update: (unstable) analytic hamiltonian - (dissipative) stabilization.
  delta = ham - diss;
  
  %------------------------------------------------------------------------
  % Reshape output into vector format and negate for RHS of ODE.
  ydot = -delta(:);

end